////////////////////////////////////////////////////////////////////// 
// merlin/MerlinFamily.h 
// (c) 2000-2001 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Monday December 17, 2001
// 
 
#ifndef __FAMILY_ANALYSIS_H__
#define __FAMILY_ANALYSIS_H__

// Task specific modules
#include "MerlinHaplotype.h"
#include "MerlinSimwalk2.h"
#include "MerlinKinship.h"
#include "MerlinMatrix.h"
#include "MerlinIBD.h"
#include "VarianceComponents.h"
#include "KAC.h"

// Stuff that gets used all the time
#include "Conquer.h"
#include "TreeInfo.h"

class FamilyAnalysis
   {
   public:
      FamilyAnalysis::FamilyAnalysis(Pedigree & pedigree);
      FamilyAnalysis::~FamilyAnalysis();

      void SetupGlobals();
      int  SetupMap(int chromosome = -1);

      void SetupFiles();
      void CloseFiles();

      bool SelectFamily(Family * f);
      void Analyse();

      void ShowInformation();
      void ShowLODs();

      void SkipFamily();
      void ScoreNPL();
      void ScoreSinglepoint();
      bool ScoreConditionals();
      void ScanChromosome();
      void SaveResults(KAC & kac, int index);

      StringArray labels;
      Vector      analysisPositions;

      int      nplCount;
      Vector   nplMean, nplInv;
      Matrix   information;
      Tree    *npl;

      IntArray markers;
      int      markerCount;
      Vector   markerPositions;

      IntArray informativeMarkers;
      Vector   informativePositions;
      Vector   informationScores;
      Vector   informationBounds;
      IntArray informationIndex;
      int      informativeCount;

      InheritanceTree * singlepoint;
      Tree            * right;
      Vector          rightScale, leftScale;

      Mantra     mantra;
      Pedigree & ped;
      Family   * family;
      int        serial;

      // What size families are we analysing
      static int  maxBits;

      // Analysis positions
      static int    steps_per_interval;
      static double maxDistance;
      static double minDistance;
      static double grid;
      static double stop;
      static double start;

      // Flags for selecting analyses to perform
      static bool findErrors;
      static bool estimateIBD;
      static bool estimateKinship;
      static bool estimateInformation;
      static bool estimateMatrices;
      static bool storeKinship;

      // Flags for controlling haplotyping
      static bool bestHaplotype;
      static int  sampledHaplotypes;
      static bool allHaplotypes;

      // Flags for controlling linkage analyses
      static bool nplAll;
      static bool nplPairs;
      static bool nplQtl;
      static bool nplDeviates;

      // Flags for controlling output
      static bool quietOutput;
      static bool useMarkerNames;
      static bool simwalk2;

      // Flags for controlling multipoint calculations
      static bool zeroRecombination;
      static bool oneRecombination;
      static bool twoRecombinations;
      static bool threeRecombinations;
      static bool twopoint;

      // Flags for controlling performance
      static bool useSwap;

      // Internal flags to minimize useless calculations
      static bool multipoint;
      static bool scanChromosome;
      static bool betweenMarkers;

      // These functions provide for buffered output and reduce clutter
      void PrintHeader();
      void PrintMessage(const char * msg, ...);
      void ProgressReport(const char * msg, int done = 1, int total = 1);
      void CleanMessages();

      // Bounds for rescaling inheritance vectors
      double lowBound, rescale;

      // Specialized engines
      MerlinMatrix       matrix;
      MerlinKinship      kinship;
      MerlinIBD          ibd;
      MerlinHaplotype    haplo;
      KAC                kac;
      TreeInfo           stats;
      NPLforSimwalk2     hybrid;
      VarianceComponents vc;

      // Flag to decide whether approximations for very sparse trees are in use
      bool useSparse;

   private:
      // Used for managing output buffering
      int  printHeader, cleanOutput;

      // Calculate left conditional & multipoint probabilities
      void WalkLeft(Multipoint & left, Multipoint & leftMarker,
                    Multipoint & inheritance, int marker);

      // Error checking routine
      void ErrorCheck(Multipoint & left, Multipoint & full, int marker);

      // Output file listing possible errors
      void OpenErrorFile();
      FILE * errorfile;

      // Functions for handling maps with zero recombination
      void SortInformation(int start, int end);
      bool ZeroRecombination();

      static void ValidateMarker(MarkerInfo * info);
   };

#endif

 
